#! /bin/sh
### BEGIN INIT INFO
# Provides:          mountkernfs
# Required-Start:
# Required-Stop:
# Should-Start:      glibc
# Default-Start:     S
# Default-Stop:
# Short-Description: Mount kernel virtual file systems.
# Description:       Mount initial set of virtual filesystems the kernel
#                    provides and that are required by everything.
### END INIT INFO

PATH=/sbin:/bin
. /lib/init/vars.sh
. /lib/init/tmpfs.sh

. /lib/lsb/init-functions
. /lib/init/mount-functions.sh

do_start () {
	# Needed to determine if root is being mounted read-only.
	read_fstab

	#
	# Get some writable area available before the root is checked
	# and remounted.
	#
	RW_OPT=
	[ "${RW_SIZE:=$TMPFS_SIZE}" ] && RW_OPT=",size=$RW_SIZE"
	[ "${RW_MODE:=$TMPFS_MODE}" ] && RW_OPT="$RW_OPT,mode=$RW_MODE"
	domount tmpfs shmfs /lib/init/rw tmpfs "-onosuid$RW_OPT"
	touch /lib/init/rw/.ramfs

	RUN_OPT=
	[ "${RUN_SIZE:=$TMPFS_SIZE}" ] && RUN_OPT=",size=$RUN_SIZE"
	[ "${RUN_MODE:=$TMPFS_MODE}" ] && RUN_OPT="$RUN_OPT,mode=$RUN_MODE"
	domount tmpfs shmfs /run tmpfs "-onosuid$RUN_OPT"
	touch /run/.ramfs

	# Make lock directory as the replacement for /var/lock
	mkdir --mode=755 /run/lock

	# Mount /run/lock as tmpfs if enabled.  This prevents user DoS
	# of /run by filling /run/lock at the expense of using an
	# additional tmpfs.
	LOCK_OPT=
	[ "${LOCK_SIZE:=$TMPFS_SIZE}" ] && LOCK_OPT=",size=$LOCK_SIZE"
	[ "${LOCK_MODE:=$TMPFS_MODE}" ] && LOCK_OPT="$LOCK_OPT,mode=$LOCK_MODE"
	if [ yes = "$RAMLOCK" ] ; then
		domount tmpfs shmfs /run/lock tmpfs "-onodev,noexec,nosuid$LOCK_OPT"
	else
		chmod "$LOCK_MODE" /run/lock
	fi

	touch /run/lock/.ramfs

	# If /tmp is a symlink, make sure the linked-to directory exists.
	if [ -L /tmp ] && [ ! -d /tmp ]; then
		TMPPATH="$(readlink /tmp)"
		mkdir -p --mode=755 "$TMPPATH"
	fi

	# If root is read only, default to mounting a tmpfs on /tmp,
	# unless one is due to be mounted from fstab.
	if [ rw != "$rootmode" ]; then
		if will_mount /tmp ; then
			:
		else
			RAMTMP="yes"
		fi
	fi

	# Mount /tmp as tmpfs if enabled.
	TMP_OPT=
	[ "${TMP_SIZE:=$TMPFS_SIZE}" ] && TMP_OPT=",size=$TMP_SIZE"
	[ "${TMP_MODE:=$TMPFS_MODE}" ] && TMP_OPT="$TMP_OPT,mode=$TMP_MODE"
	if [ yes = "$RAMTMP" ] ; then
		domount tmpfs shmfs /tmp tmpfs "-onodev,nosuid$TMP_OPT"
	else
		if [ rw = "$rootmode" ]; then
			chmod "$TMP_MODE" /tmp
		fi
	fi

	# Make pidfile omit directory for sendsigs
	# Note: /run transition: Move from /lib/init/rw to /run.
	# /lib/init/rw will be removed following transition of
	# /lib/init/rw users to /run.
	mkdir --mode=755 /lib/init/rw/sendsigs.omit.d/
	mkdir --mode=755 /run/sendsigs.omit.d/

	#
	# Mount proc filesystem on /proc
	#
	domount proc "" /proc proc "-onodev,noexec,nosuid"

	#
	# Mount sysfs on /sys
	#
	# Only mount sysfs if it is supported (kernel >= 2.6)
	if grep -E -qs "sysfs\$" /proc/filesystems
	then
		domount sysfs "" /sys sysfs "-onodev,noexec,nosuid"
	fi
}

case "$1" in
  "")
	echo "Warning: mountkernfs should be called with the 'start' argument." >&2
	do_start
	;;
  start)
	do_start
	;;
  restart|reload|force-reload)
	echo "Error: argument '$1' not supported" >&2
	exit 3
	;;
  stop)
	# No-op
	;;
  *)
	echo "Usage: mountkernfs [start|stop]" >&2
	exit 3
	;;
esac
